@@ -51,6 +51,7 @@ static gboolean RUNNING_UNDER_GDM = FALSE;
 
 static gboolean gdm_running = FALSE;
 static int GdmMinimalUID = 100;
+static char *GdmSoundProgram = NULL;
 
 static GladeXML *xml;
 
@@ -137,16 +138,18 @@ check_update_error:
 		GtkWidget *setup_dialog = glade_helper_get
 			(xml, "setup_dialog", GTK_TYPE_WINDOW);
 		GtkWidget *dlg =
-			gtk_message_dialog_new (GTK_WINDOW (setup_dialog),
+			ve_hig_dialog_new (GTK_WINDOW (setup_dialog),
 						GTK_DIALOG_MODAL | 
 						GTK_DIALOG_DESTROY_WITH_PARENT,
 						GTK_MESSAGE_ERROR,
 						GTK_BUTTONS_OK,
+						FALSE /* markup */,
 						_("An error occured while "
 						  "trying to contact the "
 						  "login screens.  Not all "
 						  "updates may have taken "
-						  "effect."));
+						  "effect."),
+						/* avoid warning */ "%s", "");
 		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
@@ -376,7 +379,6 @@ timeout_remove_all (void)
 	timeout_widgets = NULL;
 
 	for (li = list; li != NULL; li = li->next) {
-		g_print ("Removing %p!\n", li->data);
 		timeout_remove (li->data);
 		li->data = NULL;
 	}
@@ -407,6 +409,74 @@ setup_notify_toggle (const char *name,
 			  G_CALLBACK (toggle_toggled), NULL);
 }
 
+static void
+toggle_toggled_sensitivity_positive (GtkWidget *toggle, GtkWidget *depend)
+{
+	gtk_widget_set_sensitive (depend, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (toggle)));
+}
+
+static void
+setup_sensitivity_positive_toggle (const char *name,
+				   const char *depend_name)
+{
+	GtkWidget *toggle = glade_helper_get (xml, name,
+					      GTK_TYPE_TOGGLE_BUTTON);
+	GtkWidget *depend = glade_helper_get (xml, depend_name,
+					      GTK_TYPE_WIDGET);
+
+	toggle_toggled_sensitivity_positive (toggle, depend);
+
+	g_signal_connect (G_OBJECT (toggle), "toggled",
+			  G_CALLBACK (toggle_toggled_sensitivity_positive), 
+			  depend);
+}
+
+static const char *
+get_root_user (void)
+{
+	static char *root_user = NULL;
+	struct passwd *pwent;
+
+	if (root_user != NULL)
+		return root_user;
+
+	pwent = getpwuid (0);
+	if (pwent == NULL) /* huh? */
+		root_user = g_strdup ("root");
+	else
+		root_user = g_strdup (pwent->pw_name);
+	return root_user;
+}
+
+static void
+root_not_allowed (GtkWidget *entry)
+{
+	static gboolean warned = FALSE;
+	const char *text;
+
+	if (warned)
+		return;
+
+	text = gtk_entry_get_text (GTK_ENTRY (entry));
+
+	if ( ! ve_string_empty (text) &&
+	    strcmp (text, get_root_user ()) == 0) {
+		GtkWidget *dlg = 
+			ve_hig_dialog_new (NULL /* parent */,
+					   GTK_DIALOG_MODAL /* flags */,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   FALSE /* markup */,
+					   _("Autologin or timed login to the superuser (root) account is not allowed."),
+					   /* avoid warning */ "%s", "");
+		if (RUNNING_UNDER_GDM)
+			setup_cursor (GDK_LEFT_PTR);
+		gtk_dialog_run (GTK_DIALOG (dlg));
+		gtk_widget_destroy (dlg);
+		warned = TRUE;
+	}
+}
+
 static void
 setup_user_combo (const char *name, const char *key)
 {
@@ -416,6 +486,7 @@ setup_user_combo (const char *name, const char *key)
 	struct passwd *pwent;
 	char *str;
 	int cnt;
+	time_t time_started;
 
 	str = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE), key);
 
@@ -425,6 +496,8 @@ setup_user_combo (const char *name, const char *key)
 	if ( ! ve_string_empty (str))
 		users = g_list_append (users, g_strdup (str));
 
+	time_started = time (NULL);
+
 	setpwent ();
 
 	pwent = getpwent();
@@ -437,9 +510,10 @@ setup_user_combo (const char *name, const char *key)
 			users = g_list_append (users,
 					       g_strdup (pwent->pw_name));
 			/* FIXME: fix properly, see bug #111830 */
-			if (cnt >= 50) {
+			if (cnt >= 100 ||
+			    time_started + 5 <= time (NULL)) {
 				users = g_list_append (users,
-						       g_strdup ("..."));
+						       g_strdup (_("Too many users to list here...")));
 				break;
 			}
 		}
@@ -460,6 +534,8 @@ setup_user_combo (const char *name, const char *key)
 
 	g_signal_connect (G_OBJECT (entry), "changed",
 			  G_CALLBACK (entry_changed), NULL);
+	g_signal_connect (G_OBJECT (entry), "changed",
+			  G_CALLBACK (root_not_allowed), NULL);
 
 	g_list_foreach (users, (GFunc)g_free, NULL);
 	g_list_free (users);
@@ -490,6 +566,27 @@ setup_intspin (const char *name,
 			  G_CALLBACK (intspin_changed), NULL);
 }
 
+static void
+setup_notify_entry (const char *name,
+		       const char *key)
+{
+	GtkWidget *entry = glade_helper_get (xml, name, GTK_TYPE_ENTRY);
+	char *val;
+
+	val = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE), key);
+
+	g_object_set_data_full (G_OBJECT (entry),
+				"key", g_strdup (key),
+				(GDestroyNotify) g_free);
+
+	gtk_entry_set_text (GTK_ENTRY (entry), ve_sure_string (val));
+
+	g_signal_connect (G_OBJECT (entry), "changed",
+			  G_CALLBACK (entry_changed), NULL);
+
+	g_free (val);
+}
+
 static gboolean
 greeter_toggle_timeout (GtkWidget *toggle)
 {
@@ -664,8 +761,6 @@ greeter_entry_untranslate_timeout (GtkWidget *entry)
 	const char *text;
 	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
-	g_print ("Timeout %p!\n", entry);
-
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
 
 	ve_config_delete_translations (config, key);
@@ -682,7 +777,6 @@ greeter_entry_untranslate_timeout (GtkWidget *entry)
 static void
 greeter_entry_untranslate_changed (GtkWidget *entry)
 {
-	g_print ("Changed %p!\n", entry);
 	run_timeout (entry, 500, greeter_entry_untranslate_timeout);
 }
 
@@ -847,7 +941,7 @@ xdmcp_toggled (GtkWidget *toggle, gpointer data)
 {
 	GtkWidget *frame = data;
 
-	gtk_widget_set_sensitive (GTK_BIN (frame)->child,
+	gtk_widget_set_sensitive (frame,
 				  GTK_TOGGLE_BUTTON (toggle)->active);
 }
 
@@ -855,17 +949,19 @@ static void
 setup_xdmcp_support (void)
 {
 	GtkWidget *xdmcp_toggle = glade_helper_get (xml, "enable_xdmcp", GTK_TYPE_TOGGLE_BUTTON);
-	GtkWidget *xdmcp_frame = glade_helper_get (xml, "xdmcp_frame", GTK_TYPE_FRAME);
+	GtkWidget *xdmcp_frame = glade_helper_get (xml, "xdmcp_frame", GTK_TYPE_WIDGET);
+	GtkWidget *xdmcp_vbox = glade_helper_get (xml, "xdmcp_vbox", GTK_TYPE_WIDGET);
+	GtkWidget *no_xdmcp_label = glade_helper_get (xml, "no_xdmcp_label", GTK_TYPE_WIDGET);
 
 #ifndef HAVE_LIBXDMCP
-	gtk_widget_show (glade_helper_get (xml, "no_xdmcp_label", GTK_TYPE_LABEL));
-	gtk_widget_set_sensitive (xdmcp_toggle, FALSE);
-	gtk_widget_set_sensitive (xdmcp_frame, FALSE);
-#else /* ! HAVE_LIBXDMCP */
-	gtk_widget_hide (glade_helper_get (xml, "no_xdmcp_label", GTK_TYPE_LABEL));
+	gtk_widget_show (no_xdmcp_label);
+	gtk_widget_hide (xdmcp_vbox);
+#else /* HAVE_LIBXDMCP */
+	gtk_widget_hide (no_xdmcp_label);
+	gtk_widget_show (xdmcp_vbox);
 #endif /* HAVE_LIBXDMCP */
 
-	gtk_widget_set_sensitive (GTK_BIN (xdmcp_frame)->child, 
+	gtk_widget_set_sensitive (xdmcp_frame, 
 				  GTK_TOGGLE_BUTTON (xdmcp_toggle)->active);
 
 	g_signal_connect (G_OBJECT (xdmcp_toggle), "toggled",
@@ -873,6 +969,210 @@ setup_xdmcp_support (void)
 			  xdmcp_frame);
 }
 
+static void
+test_sound (GtkWidget *button, gpointer data)
+{
+	GtkEntry *entry = GTK_ENTRY (data);
+	const char *file = gtk_entry_get_text (GTK_ENTRY (entry));
+	const char *argv[3];
+
+	if (ve_string_empty (file) ||
+	    access (file, R_OK) != 0 ||
+	    ve_string_empty (GdmSoundProgram))
+	       return;
+
+	argv[0] = GdmSoundProgram;
+	argv[1] = file;
+	argv[2] = NULL;
+
+	g_spawn_async ("/" /* working directory */,
+		       (char **)argv,
+		       NULL /* envp */,
+		       0 /* flags */,
+		       NULL /* child setup */,
+		       NULL /* user data */,
+		       NULL /* child pid */,
+		       NULL /* error */);
+}
+
+static gboolean
+module_compare (const char *mod1, const char *mod2)
+{
+	char *base1;
+	char *base2;
+	char *p;
+	gboolean ret;
+
+	/* first cannonify the names */
+	base1 = g_path_get_basename (mod1);
+	base2 = g_path_get_basename (mod2);
+	if (strncmp (base1, "lib", 3) == 0)
+		strcpy (base1, &base1[3]);
+	if (strncmp (base2, "lib", 3) == 0)
+		strcpy (base2, &base2[3]);
+	p = strstr (base1, ".so");
+	if (p != NULL)
+		*p = '\0';
+	p = strstr (base2, ".so");
+	if (p != NULL)
+		*p = '\0';
+
+	ret = (strcmp (base1, base2) == 0);
+
+	g_free (base1);
+	g_free (base2);
+
+	return ret;
+}
+
+static gboolean
+modules_list_contains (const char *modules_list, const char *module)
+{
+	char **vec;
+	int i;
+
+	if (ve_string_empty (modules_list))
+		return FALSE;
+
+	vec = g_strsplit (modules_list, ":", -1);
+	if (vec == NULL)
+		return FALSE;
+
+	for (i = 0; vec[i] != NULL; i++) {
+		if (module_compare (vec[i], module)) {
+			g_strfreev (vec);
+			return TRUE;
+		}
+	}
+
+	g_strfreev (vec);
+	return FALSE;
+}
+
+static char *
+modules_list_remove (char *modules_list, const char *module)
+{
+	char **vec;
+	GString *str;
+	char *sep = "";
+	int i;
+
+	if (ve_string_empty (modules_list))
+		return g_strdup ("");
+
+	vec = g_strsplit (modules_list, ":", -1);
+	if (vec == NULL)
+		return g_strdup ("");
+
+	str = g_string_new (NULL);
+
+	for (i = 0; vec[i] != NULL; i++) {
+		if ( ! module_compare (vec[i], module)) {
+			g_string_append (str, sep);
+			sep = ":";
+			g_string_append (str, vec[i]);
+		}
+	}
+
+	g_strfreev (vec);
+
+	return g_string_free (str, FALSE);
+}
+
+static char *
+modules_list_add (char *modules_list, const char *module)
+{
+	char *n;
+	if (ve_string_empty (modules_list))
+		n = g_strdup (module);
+	else
+		n = g_strconcat (modules_list, ":", module, NULL);
+	g_free (modules_list);
+	return n;
+}
+
+static void
+acc_modules_toggled (GtkWidget *toggle, gpointer data)
+{
+	gboolean add_gtk_modules = ve_config_get_bool (ve_config_get (GDM_CONFIG_FILE),
+						       GDM_KEY_ADD_GTK_MODULES);
+	char *modules_list = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+						   GDM_KEY_GTK_MODULES_LIST);
+
+	/* first whack the modules from the list */
+	modules_list = modules_list_remove (modules_list, "gail");
+	modules_list = modules_list_remove (modules_list, "atk-bridge");
+	modules_list = modules_list_remove (modules_list, "dwellmouselistener");
+	modules_list = modules_list_remove (modules_list, "keymouselistener");
+
+	if (GTK_TOGGLE_BUTTON (toggle)->active) {
+		if ( ! add_gtk_modules) {
+			g_free (modules_list);
+			modules_list = NULL;
+		}
+
+		modules_list = modules_list_add (modules_list, "gail");
+		modules_list = modules_list_add (modules_list, "atk-bridge");
+		modules_list = modules_list_add (modules_list, EXPANDED_LIBDIR "/gtk-2.0/modules/libkeymouselistener");
+		modules_list = modules_list_add (modules_list, EXPANDED_LIBDIR "/gtk-2.0/modules/libdwellmouselistener");
+
+		add_gtk_modules = TRUE;
+	}
+
+	if (ve_string_empty (modules_list))
+		add_gtk_modules = FALSE;
+
+	ve_config_set_string (ve_config_get (GDM_CONFIG_FILE),
+			      GDM_KEY_GTK_MODULES_LIST,
+			      ve_sure_string (modules_list));
+	ve_config_set_bool (ve_config_get (GDM_CONFIG_FILE),
+			    GDM_KEY_ADD_GTK_MODULES,
+			    add_gtk_modules);
+
+	ve_config_save (ve_config_get (GDM_CONFIG_FILE), FALSE /*force */);
+
+	g_free (modules_list);
+
+	update_key (GDM_KEY_GTK_MODULES_LIST);
+	update_key (GDM_KEY_ADD_GTK_MODULES);
+}
+
+static void
+setup_accessibility_support (void)
+{
+	GtkWidget *acc_modules = glade_helper_get (xml, "acc_modules", GTK_TYPE_TOGGLE_BUTTON);
+	GtkWidget *acc_sound_test = glade_helper_get (xml, "acc_sound_test", GTK_TYPE_BUTTON);
+	GtkWidget *acc_sound_file_entry = glade_helper_get (xml, "acc_sound_file_entry", GTK_TYPE_ENTRY);
+	GtkWidget *acc_sound_file = glade_helper_get (xml, "acc_sound_file", GNOME_TYPE_FILE_ENTRY);
+
+	gboolean add_gtk_modules = ve_config_get_bool (ve_config_get (GDM_CONFIG_FILE),
+						       GDM_KEY_ADD_GTK_MODULES);
+	char *modules_list = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+						   GDM_KEY_GTK_MODULES_LIST);
+
+	if (add_gtk_modules &&
+	    modules_list_contains (modules_list, "gail") &&
+	    modules_list_contains (modules_list, "atk-bridge") &&
+	    modules_list_contains (modules_list, "dwellmouselistener") &&
+	    modules_list_contains (modules_list, "keymouselistener")) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (acc_modules),
+					      TRUE);
+	} else {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (acc_modules),
+					      FALSE);
+	}
+
+	g_signal_connect (G_OBJECT (acc_modules), "toggled",
+			  G_CALLBACK (acc_modules_toggled),
+			  NULL);
+	g_signal_connect (G_OBJECT (acc_sound_test), "clicked",
+			  G_CALLBACK (test_sound),
+			  acc_sound_file_entry);
+
+	if (access ("/usr/share/sounds", F_OK) == 0)
+		gnome_file_entry_set_default_path (GNOME_FILE_ENTRY (acc_sound_file), "/usr/share/sounds/");
+}
+
 static void
 background_toggled (void)
 {
@@ -982,13 +1282,28 @@ selection_changed (GtkTreeSelection *selection, gpointer data)
 					       GTK_TYPE_IMAGE);
         GtkWidget *no_preview = glade_helper_get (xml, "gg_theme_no_preview",
 						  GTK_TYPE_WIDGET);
+	GtkWidget *del_button = glade_helper_get (xml, "gg_delete_theme",
+						  GTK_TYPE_BUTTON);
         char *screenshot;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GValue value = {0, };
 
-	if ( ! gtk_tree_selection_get_selected (selection, &model, &iter))
+	if ( ! gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		gtk_widget_set_sensitive (del_button, FALSE);
 		return;
+	}
+
+	gtk_widget_set_sensitive (del_button, TRUE);
+
+	gtk_tree_model_get_value (model, &iter,
+				  THEME_COLUMN_SELECTED,
+				  &value);
+	/* Do not allow deleting of selected theme */
+	if (g_value_get_boolean (&value)) {
+		gtk_widget_set_sensitive (del_button, FALSE);
+	}
+	g_value_unset (&value);
 
 	gtk_tree_model_get_value (model, &iter,
 				  THEME_COLUMN_AUTHOR,
@@ -1303,6 +1618,55 @@ find_tar (void)
 	return g_strdup ("/bin/tar");
 }
 
+static char *
+find_chmod (void)
+{
+	char *chmod_prog;
+	char *try[] = {
+		"/bin/chmod",
+		"/sbin/chmod",
+		"/usr/bin/chmod",
+		"/usr/sbin/chmod",
+		NULL };
+	int i;
+
+	chmod_prog = g_find_program_in_path ("chmod");
+	if (chmod_prog != NULL)
+		return chmod_prog;
+
+	for (i = 0; try[i] != NULL; i++) {
+		if (access (try[i], X_OK) == 0)
+			return g_strdup (try[i]);
+	}
+	/* Hmmm, fallback */
+	return g_strdup ("/bin/chmod");
+}
+
+static char *
+find_chown (void)
+{
+	char *chown_prog;
+	char *try[] = {
+		"/bin/chown",
+		"/sbin/chown",
+		"/usr/bin/chown",
+		"/usr/sbin/chown",
+		NULL };
+	int i;
+
+	chown_prog = g_find_program_in_path ("chown");
+	if (chown_prog != NULL)
+		return chown_prog;
+
+	for (i = 0; try[i] != NULL; i++) {
+		if (access (try[i], X_OK) == 0)
+			return g_strdup (try[i]);
+	}
+	/* Hmmm, fallback */
+	return g_strdup ("/bin/chown");
+}
+
+
 static char *
 get_the_dir (FILE *fp, char **error)
 {
@@ -1471,10 +1835,9 @@ dir_exists (const char *parent, const char *dir)
 }
 
 static void
-install_ok (GtkWidget *button, gpointer data)
+install_response (GtkWidget *chooser, gint response, gpointer data)
 {
-	GtkFileSelection *fs = GTK_FILE_SELECTION (data);
-	GtkListStore *store = g_object_get_data (G_OBJECT (fs), "ListStore");
+	GtkListStore *store = data;
 	GtkWidget *theme_list = glade_helper_get (xml, "gg_theme_list",
 						  GTK_TYPE_TREE_VIEW);
 	char *filename, *dir, *untar_cmd, *theme_dir, *cwd;
@@ -1484,19 +1847,25 @@ install_ok (GtkWidget *button, gpointer data)
 	DIR *dp;
 	gboolean success = FALSE;
 
+	if (response != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (chooser);
+		return;
+	}
+
 	cwd = g_get_current_dir ();
 	theme_dir = get_theme_dir ();
 
-	filename = g_strdup (gtk_file_selection_get_filename (fs));
+	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
 	if (filename == NULL) {
 		GtkWidget *dlg =
-			gtk_message_dialog_new (GTK_WINDOW (fs),
-						GTK_DIALOG_MODAL | 
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("No file selected"));
-		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+			ve_hig_dialog_new (GTK_WINDOW (chooser),
+					   GTK_DIALOG_MODAL | 
+					   GTK_DIALOG_DESTROY_WITH_PARENT,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   FALSE /* markup */,
+					   _("No file selected"),
+					   /* avoid warning */ "%s", "");
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 		g_free (cwd);
@@ -1517,15 +1886,15 @@ install_ok (GtkWidget *button, gpointer data)
 
 	if (dir == NULL) {
 		GtkWidget *dlg =
-			gtk_message_dialog_new (GTK_WINDOW (fs),
-						GTK_DIALOG_MODAL | 
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("Not a theme archive\n"
-						  "Details: %s"),
-						error);
-		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+			ve_hig_dialog_new (GTK_WINDOW (chooser),
+					   GTK_DIALOG_MODAL | 
+					   GTK_DIALOG_DESTROY_WITH_PARENT,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   FALSE /* markup */,
+					   _("Not a theme archive"),
+					   _("Details: %s"),
+					   error);
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 		g_free (filename);
@@ -1535,21 +1904,42 @@ install_ok (GtkWidget *button, gpointer data)
 	}
 
 	if (dir_exists (theme_dir, dir)) {
-		char *fname = g_filename_to_utf8 (dir, -1, NULL, NULL, NULL);
+		char *fname = ve_filename_to_utf8 (dir);
+		char *s;
+		GtkWidget *button;
+		GtkWidget *dlg;
+
 		/* FIXME: if exists already perhaps we could also have an
 		 * option to change the dir name */
-		GtkWidget *dlg =
-			gtk_message_dialog_new
-			(GTK_WINDOW (fs),
+		s = g_strdup_printf (_("Theme directory '%s' seems to be already "
+				       "installed, install again anyway?"),
+				     fname);
+		dlg = ve_hig_dialog_new
+			(GTK_WINDOW (chooser),
 			 GTK_DIALOG_MODAL | 
 			 GTK_DIALOG_DESTROY_WITH_PARENT,
 			 GTK_MESSAGE_QUESTION,
-			 GTK_BUTTONS_YES_NO,
-			 _("Theme directory '%s' seems to be already "
-			   "installed, install again anyway?"),
-			 fname);
-		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+			 GTK_BUTTONS_NONE,
+			 FALSE /* markup */,
+			 s,
+			 /* avoid warning */ "%s", "");
 		g_free (fname);
+		g_free (s);
+
+		button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+		gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, GTK_RESPONSE_NO);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+
+		button = gtk_button_new_from_stock ("_Install Anyway");
+		gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, GTK_RESPONSE_YES);
+		GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+		gtk_widget_show (button);
+
+		gtk_dialog_set_default_response (GTK_DIALOG (dlg),
+						 GTK_RESPONSE_YES);
+
+		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		if (gtk_dialog_run (GTK_DIALOG (dlg)) != GTK_RESPONSE_YES) {
 			gtk_widget_destroy (dlg);
 			g_free (filename);
@@ -1566,34 +1956,42 @@ install_ok (GtkWidget *button, gpointer data)
 	if (chdir (theme_dir) == 0) {
 		if (system (untar_cmd) == 0) {
 			char *cmd;
+			char *quoted = g_shell_quote (dir);
+			char *chown = find_chown ();
+			char *chmod = find_chmod ();
 			success = TRUE;
 
 			/* HACK! */
-			cmd = g_strdup_printf ("/bin/chown -R root.root %s", dir);
+			cmd = g_strdup_printf ("%s -R root.root %s", chown, quoted);
 			system (cmd);
 			g_free (cmd);
 
-			cmd = g_strdup_printf ("/bin/chmod -R a+r %s", dir);
+			cmd = g_strdup_printf ("%s -R a+r %s", chmod, quoted);
 			system (cmd);
 			g_free (cmd);
 
-			cmd = g_strdup_printf ("/bin/chmod a+x %s", dir);
+			cmd = g_strdup_printf ("%s a+x %s", chmod, quoted);
 			system (cmd);
 			g_free (cmd);
+
+			g_free (quoted);
+			g_free (chown);
+			g_free (chmod);
 		}
 		chdir (cwd);
 	}
 
 	if ( ! success) {
 		GtkWidget *dlg =
-			gtk_message_dialog_new (GTK_WINDOW (fs),
-						GTK_DIALOG_MODAL | 
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("Some error occured when "
-						  "installing the theme"));
-		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+			ve_hig_dialog_new (GTK_WINDOW (chooser),
+					   GTK_DIALOG_MODAL | 
+					   GTK_DIALOG_DESTROY_WITH_PARENT,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   FALSE /* markup */,
+					   _("Some error occured when "
+					     "installing the theme"),
+					   /* avoid warning */ "%s", "");
 		gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 	}
@@ -1620,36 +2018,157 @@ install_ok (GtkWidget *button, gpointer data)
 	g_free (cwd);
 	g_free (theme_dir);
 
-	gtk_widget_destroy (GTK_WIDGET (fs));
+	gtk_widget_destroy (GTK_WIDGET (chooser));
 }
 
 static void
 install_new_theme (GtkWidget *button, gpointer data)
 {
 	GtkListStore *store = data;
-	static GtkWidget *fs = NULL;
+	static GtkWidget *chooser = NULL;
 	GtkWidget *setup_dialog;
 
-	if (fs != NULL) {
-		gtk_window_present (GTK_WINDOW (fs));
+	if (chooser != NULL) {
+		gtk_window_present (GTK_WINDOW (chooser));
 		return;
 	}
        
 	setup_dialog = glade_helper_get (xml, "setup_dialog", GTK_TYPE_WINDOW);
 	
-	fs = gtk_file_selection_new (_("Select new theme archive to install"));
-	gtk_window_set_transient_for (GTK_WINDOW (fs),
-				      GTK_WINDOW (setup_dialog));
-	g_object_set_data (G_OBJECT (fs), "ListStore", store);
-	g_signal_connect (G_OBJECT (fs), "destroy",
-			  G_CALLBACK (gtk_widget_destroyed), &fs);
-	g_signal_connect (GTK_FILE_SELECTION (fs)->ok_button, "clicked",
-			  G_CALLBACK (install_ok), fs);
-	g_signal_connect_swapped (GTK_FILE_SELECTION (fs)->cancel_button,
-				  "clicked",
-				  G_CALLBACK (gtk_widget_destroy), fs);
+	chooser = gtk_file_chooser_dialog_new (_("Select new theme archive to install"),
+					       GTK_WINDOW (setup_dialog),
+					       GTK_FILE_CHOOSER_ACTION_OPEN,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       _("_Install"), GTK_RESPONSE_OK,
+					       NULL);
+	
+	g_signal_connect (G_OBJECT (chooser), "destroy",
+			  G_CALLBACK (gtk_widget_destroyed), &chooser);
+	g_signal_connect (G_OBJECT (chooser), "response",
+			  G_CALLBACK (install_response), store);
 
-	gtk_widget_show (fs);
+	gtk_widget_show (chooser);
+}
+
+static void
+delete_theme (GtkWidget *button, gpointer data)
+{
+	GtkListStore *store = data;
+	GtkWidget *theme_list;
+	GtkWidget *setup_dialog;
+	GtkWidget *del_button;
+	GtkTreeSelection *selection;
+        char *dir, *name;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GValue value = {0, };
+	GtkWidget *dlg;
+	char *s;
+
+	setup_dialog = glade_helper_get (xml, "setup_dialog", GTK_TYPE_WINDOW);
+	theme_list = glade_helper_get (xml, "gg_theme_list",
+				       GTK_TYPE_TREE_VIEW);
+	del_button = glade_helper_get (xml, "gg_delete_theme",
+				       GTK_TYPE_BUTTON);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (theme_list));
+
+	if ( ! gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		/* should never get here since the button shuld not be
+		 * enabled */
+		return;
+	}
+
+	gtk_tree_model_get_value (model, &iter,
+				  THEME_COLUMN_SELECTED,
+				  &value);
+	/* Do not allow deleting of selected theme */
+	if (g_value_get_boolean (&value)) {
+		/* should never get here since the button shuld not be
+		 * enabled */
+		g_value_unset (&value);
+		return;
+	}
+	g_value_unset (&value);
+
+	gtk_tree_model_get_value (model, &iter,
+				  THEME_COLUMN_NAME,
+				  &value);
+	name = g_strdup (g_value_get_string (&value));
+	g_value_unset (&value);
+
+	gtk_tree_model_get_value (model, &iter,
+				  THEME_COLUMN_DIR,
+				  &value);
+	dir = g_strdup (g_value_get_string (&value));
+	g_value_unset (&value);
+
+	s = g_strdup_printf (_("Do you really wish to remove theme '%s' from the system?"),
+			     name);
+	dlg = ve_hig_dialog_new
+		(GTK_WINDOW (setup_dialog),
+		 GTK_DIALOG_MODAL | 
+		 GTK_DIALOG_DESTROY_WITH_PARENT,
+		 GTK_MESSAGE_QUESTION,
+		 GTK_BUTTONS_NONE,
+		 FALSE /* markup */,
+		 s,
+		 /* avoid warning */ "%s", "");
+	g_free (s);
+
+	button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
+	gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, GTK_RESPONSE_NO);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_show (button);
+
+	button = gtk_button_new_from_stock ("_Remove Theme");
+	gtk_dialog_add_action_widget (GTK_DIALOG (dlg), button, GTK_RESPONSE_YES);
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+	gtk_widget_show (button);
+	
+	gtk_dialog_set_default_response (GTK_DIALOG (dlg),
+					 GTK_RESPONSE_YES);
+
+	if (gtk_dialog_run (GTK_DIALOG (dlg)) == GTK_RESPONSE_YES) {
+		char *theme_dir = get_theme_dir ();
+		char *cwd = g_get_current_dir ();
+		if (chdir (theme_dir) == 0 &&
+		    /* this is a security sanity check, since we're doing rm -fR */
+		    strchr (dir, '/') == NULL) {
+			/* HACK! */
+			DIR *dp;
+			GtkTreeIter *select_iter = NULL;
+			char *quoted = g_shell_quote (dir);
+			char *cmd = g_strdup_printf ("/bin/rm -fR %s", quoted);
+			system (cmd);
+			g_free (cmd);
+			g_free (quoted);
+
+			/* Update the list */
+			gtk_list_store_clear (store);
+
+			dp = opendir (theme_dir);
+
+			if (dp != NULL) {
+				select_iter = read_themes (store, theme_dir, dp, 
+							   selected_theme);
+				closedir (dp);
+			}
+
+			if (select_iter != NULL) {
+				gtk_tree_selection_select_iter (selection, select_iter);
+				g_free (select_iter);
+			}
+
+		}
+		chdir (cwd);
+		g_free (cwd);
+		g_free (theme_dir);
+	}
+	gtk_widget_destroy (dlg);
+
+	g_free (name);
+	g_free (dir);
 }
 
 static void
@@ -1665,9 +2184,13 @@ setup_graphical_themes (void)
 						  GTK_TYPE_TREE_VIEW);
 	GtkWidget *button = glade_helper_get (xml, "gg_install_new_theme",
 					      GTK_TYPE_BUTTON);
+	GtkWidget *del_button = glade_helper_get (xml, "gg_delete_theme",
+						  GTK_TYPE_BUTTON);
 
 	char *theme_dir = get_theme_dir ();
 
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (theme_list), TRUE);
+
 	selected_theme = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
 					       GDM_KEY_GRAPHICAL_THEME);
 
@@ -1684,6 +2207,9 @@ setup_graphical_themes (void)
 
 	g_signal_connect (button, "clicked",
 			  G_CALLBACK (install_new_theme), store);
+	g_signal_connect (del_button, "clicked",
+			  G_CALLBACK (delete_theme), store);
+	gtk_widget_set_sensitive (del_button, FALSE);
 
 	dir = opendir (theme_dir);
 
@@ -1761,16 +2287,27 @@ dialog_response (GtkWidget *dlg, int response, gpointer data)
 		timeout_remove_all ();
 		gtk_main_quit ();
 	} else if (response == GTK_RESPONSE_HELP) {
-		/* FIXME: when help gets written connect it here */
+		GError *error = NULL;
 		GtkWidget *setup_dialog = glade_helper_get
 			(xml, "setup_dialog", GTK_TYPE_WINDOW);
-		static GtkWidget *dlg;
+		static GtkWidget *dlg = NULL;
 
 		if (dlg != NULL) {
 			gtk_window_present (GTK_WINDOW (dlg));
 			return;
 		}
+
+		if ( ! RUNNING_UNDER_GDM) {
+			gnome_help_display_uri ("ghelp:gdm", &error);
+			/* FIXME: handle errors nicer */
+			if (error == NULL)
+				return;
+			g_error_free (error);
+		}
+
+		/* fallback help dialogue */
 	
+		/* HIG compliance? */
 		dlg = gtk_message_dialog_new
 			(GTK_WINDOW (setup_dialog),
 			 GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -1783,7 +2320,9 @@ dialog_response (GtkWidget *dlg, int response, gpointer data)
 			   "will take effect immediately.\n\n"
 			   "Note that not all configuration options "
 			   "are listed here.  You may want to edit %s "
-			   "if you cannot find what you are looking for."),
+			   "if you cannot find what you are looking for.\n\n"
+			   "For complete documentation see the GNOME help browser "
+			   "Under the category GNOME/System."),
 			 GDM_CONFIG_FILE);
 		gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
 		g_signal_connect (G_OBJECT (dlg), "destroy",
@@ -1796,10 +2335,18 @@ dialog_response (GtkWidget *dlg, int response, gpointer data)
 	}
 }
 
+static void
+add_to_size_group (GtkSizeGroup *sg, const char *wid)
+{
+	GtkWidget *w = glade_helper_get (xml, wid, GTK_TYPE_WIDGET);
+	gtk_size_group_add_widget (sg, w);
+}
+
 static void
 setup_gui (void)
 {
 	GtkWidget *dialog;
+	GtkSizeGroup *sg;
 
 	xml = glade_helper_load ("gdmsetup.glade",
 				 "setup_dialog",
@@ -1811,11 +2358,70 @@ setup_gui (void)
 	g_signal_connect (G_OBJECT (dialog), "response",
 			  G_CALLBACK (dialog_response), NULL);
 
+	/* setup bold thingies */
+	glade_helper_tagify_label (xml, "greeter_cat_label", "b");
+	glade_helper_tagify_label (xml, "autologin_cat_label", "b");
+	glade_helper_tagify_label (xml, "timedlogin_cat_label", "b");
+
+	glade_helper_tagify_label (xml, "sg_logo_cat_label", "b");
+	glade_helper_tagify_label (xml, "sg_background_cat_label", "b");
+	glade_helper_tagify_label (xml, "sg_misc_cat_label", "b");
+
+	glade_helper_tagify_label (xml, "options_cat_label", "b");
+
+	glade_helper_tagify_label (xml, "gg_preview_cat_label", "b");
+	glade_helper_tagify_label (xml, "gg_author_label", "b");
+	glade_helper_tagify_label (xml, "gg_desc_label", "b");
+	glade_helper_tagify_label (xml, "gg_copyright_label", "b");
+
+	glade_helper_tagify_label (xml, "acc_options_cat_label", "b");
+
+	glade_helper_tagify_label (xml, "enable_xdmcp", "b");
+
+	setup_accessibility_support ();
 	setup_xdmcp_support ();
 	setup_background_support ();
 	setup_greeter_backselect ();
 	setup_graphical_themes ();
 
+       	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	add_to_size_group (sg, "local_greeter");
+	add_to_size_group (sg, "remote_greeter");
+	add_to_size_group (sg, "welcome");
+	add_to_size_group (sg, "remote_welcome");
+	add_to_size_group (sg, "autologin_combo");
+	add_to_size_group (sg, "timedlogin_combo");
+	add_to_size_group (sg, "timedlogin_seconds");
+	g_object_unref (G_OBJECT (sg));
+
+       	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	add_to_size_group (sg, "local_greeter_label");
+	add_to_size_group (sg, "remote_greeter_label");
+	add_to_size_group (sg, "welcome_label");
+	add_to_size_group (sg, "remote_welcome_label");
+	add_to_size_group (sg, "autologin_label");
+	add_to_size_group (sg, "timed_login_label");
+	add_to_size_group (sg, "timedlogin_seconds_label");
+	g_object_unref (G_OBJECT (sg));
+
+       	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	add_to_size_group (sg, "greeter_table");
+	add_to_size_group (sg, "autologin_table");
+	add_to_size_group (sg, "timed_login_table");
+	g_object_unref (G_OBJECT (sg));
+
+       	sg = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+	add_to_size_group (sg, "sg_use_24_clock");
+	add_to_size_group (sg, "autologin");
+	add_to_size_group (sg, "timedlogin");
+	g_object_unref (G_OBJECT (sg));
+
+	/* Note: Just setting up a notify here doesn't mean
+	   things will get updated.  You must also wire in
+	   the logic for update in the daemon and/or greeters
+	   (or chooser) or whereever this is actually being
+	   updated. */
+
 	setup_user_combo ("autologin_combo",
 			  GDM_KEY_AUTOMATICLOGIN);
 	setup_user_combo ("timedlogin_combo",
@@ -1843,6 +2449,24 @@ setup_gui (void)
 	setup_notify_toggle ("config_available",
 			     GDM_KEY_CONFIG_AVAILABLE,
 			     GDM_KEY_CONFIG_AVAILABLE /* notify_key */);
+	setup_notify_toggle ("chooser_button",
+			     GDM_KEY_CHOOSER_BUTTON,
+			     GDM_KEY_CHOOSER_BUTTON /* notify_key */);
+	setup_notify_toggle ("disallow_tcp",
+			     GDM_KEY_DISALLOWTCP,
+			     GDM_KEY_DISALLOWTCP /* notify_key */);
+
+	/* security sensitivities */
+	setup_sensitivity_positive_toggle ("sysmenu", "config_available");
+	setup_sensitivity_positive_toggle ("sysmenu", "chooser_button");
+
+	setup_greeter_toggle ("acc_beep",
+			      GDM_KEY_SOUND_ON_LOGIN);
+	setup_notify_entry ("acc_sound_file_entry",
+			    GDM_KEY_SOUND_ON_LOGIN_FILE);
+
+	/* accesibility sensitivities */
+	setup_sensitivity_positive_toggle ("acc_beep", "acc_sound_file_box");
 
 	setup_notify_toggle ("enable_xdmcp",
 			     GDM_KEY_XDMCP,
@@ -1905,8 +2529,10 @@ setup_gui (void)
 	setup_greeter_color ("sg_backcolor",
 			     GDM_KEY_BACKGROUNDCOLOR);
 
-	setup_greeter_untranslate_entry ("sg_welcome",
+	setup_greeter_untranslate_entry ("welcome",
 					 GDM_KEY_WELCOME);
+	setup_greeter_untranslate_entry ("remote_welcome",
+					 GDM_KEY_REMOTEWELCOME);
 }
 
 static gboolean
@@ -2033,6 +2659,7 @@ main (int argc, char *argv[])
 	gnome_program_init ("gdmsetup", VERSION, 
 			    LIBGNOMEUI_MODULE /* module_info */,
 			    argc, argv,
+			    GNOME_PROGRAM_STANDARD_PROPERTIES,
 			    /* *GNOME_PARAM_POPT_TABLE, options, */
 			    GNOME_PARAM_CREATE_DIRECTORIES, ! RUNNING_UNDER_GDM,
 			    NULL);
@@ -2078,12 +2705,13 @@ main (int argc, char *argv[])
 	if ( ! DOING_GDM_DEVELOPMENT &&
 	     geteuid() != 0) {
 		GtkWidget *fatal_error = 
-			gtk_message_dialog_new (NULL /* parent */,
-						GTK_DIALOG_MODAL /* flags */,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("You must be the superuser (root) to configure GDM.\n"));
-		gtk_dialog_set_has_separator (GTK_DIALOG (fatal_error), FALSE);
+			ve_hig_dialog_new (NULL /* parent */,
+					   GTK_DIALOG_MODAL /* flags */,
+					   GTK_MESSAGE_ERROR,
+					   GTK_BUTTONS_OK,
+					   FALSE /* markup */,
+					   _("You must be the superuser (root) to configure GDM."),
+					   /* avoid warning */ "%s", "");
 		if (RUNNING_UNDER_GDM)
 			setup_cursor (GDK_LEFT_PTR);
 		gtk_dialog_run (GTK_DIALOG (fatal_error));
@@ -2094,6 +2722,13 @@ main (int argc, char *argv[])
 	 * ui?  Say it ain't so.  Our config sections are SUCH A MESS */
 	GdmMinimalUID = ve_config_get_int (ve_config_get (GDM_CONFIG_FILE),
 					   GDM_KEY_MINIMALUID);
+	GdmSoundProgram = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+						GDM_KEY_SOUND_PROGRAM);
+	if (ve_string_empty (GdmSoundProgram) ||
+	    access (GdmSoundProgram, X_OK) != 0) {
+		g_free (GdmSoundProgram);
+		GdmSoundProgram = NULL;
+	}
 
 	setup_gui ();
 
